﻿local GatherMate = LibStub("AceAddon-3.0"):GetAddon("GatherMate")
local Config = GatherMate:NewModule("Config","AceConsole-3.0","AceEvent-3.0")
local Display = GatherMate:GetModule("Display")
local L = LibStub("AceLocale-3.0"):GetLocale("GatherMate", false)

--[[
	Code here for configing the mod, and making the minimap button
]]

local prof_options = {
	["always"]          = L["Always show"],
	["with_profession"] = L["Only with profession"],
	["active"]          = L["Only while tracking"],
	["never"]           = L["Never show"],
}
local prof_options2 = { -- For Gas, which doesn't have tracking as a skill
	["always"]           = L["Always show"],
	["with_profession"]  = L["Only with profession"],
	["never"]            = L["Never show"],
}
local prof_options3 = {
	["always"]          = L["Always show"],
	["active"]          = L["Only while tracking"],
	["never"]           = L["Never show"],
}

local options = {}
local db
local imported = {}
-- setup the options, we need to reference GatherMate for this
options.type = "group"
options.name = "GatherMate"
options.get = function( k ) return db[k.arg] end
options.set = function( k, v ) db[k.arg] = v; Config:UpdateConfig(); end
options.args = {}
options.plugins = {}

-- Display Settings config tree
options.args.display = {
 	type = "group",
	name = L["Display Settings"],
	order = 1,
	args = {},
}
options.args.display.args.general = {
	type = "group",
	name = L["General"],
	order = 1,
	args = {
		showGroup = {
			type = "group",
			name = L["Show Databases"],
			guiInline = true,
			order = 1,
			get = function(k) return db.show[k.arg] end,
			set = function(k, v) db.show[k.arg] = v; Config:UpdateConfig(); end,
			args = {
				desc = {
					order = 0,
					type = "description",
					name = L["Selected databases are shown on both the World Map and Minimap."],
				},
				showMinerals = {
					order = 1,
					name = L["Show Mining Nodes"],
					desc = L["Toggle showing mining nodes."],
					type = "select",
					values = prof_options,
					arg = "Mining"
				},
				showHerbs = {
					order = 2,
					name = L["Show Herbalism Nodes"],
					desc = L["Toggle showing herbalism nodes."],
					type = "select",
					values = prof_options,
					arg = "Herb Gathering"
				},
				showFishes = {
					order = 3,
					name = L["Show Fishing Nodes"],
					desc = L["Toggle showing fishing nodes."],
					type = "select",
					values = prof_options,
					arg = "Fishing"
				},
				showGases = {
					order = 4,
					name = L["Show Gas Clouds"],
					desc = L["Toggle showing gas clouds."],
					type = "select",
					values = prof_options2,
					arg = "Extract Gas"
				},
				showTreasure = {
					order = 5,
					name = L["Show Treasure Nodes"],
					desc = L["Toggle showing treasure nodes."],
					type = "select",
					values = prof_options3,
					arg = "Opening"
				},
			},
		},
		iconGroup = {
			type = "group",
			name = L["Icons"],
			guiInline = true,
			order = 2,
			args = {
				desc = {
					order = 0,
					type = "description",
					name = L["Control various aspects of node icons on both the World Map and Minimap."],
				},
				showMinimapIcons = {
					order = 1,
					name = L["Show Minimap Icons"],
					desc = L["Toggle showing Minimap icons."],
					type = "toggle",
					arg = "showMinimap",
				},
				showWorldMapIcons = {
					order = 2,
					name = L["Show World Map Icons"],
					desc = L["Toggle showing World Map icons."],
					type = "toggle",
					arg = "showWorldMap",
				},
				iconScale = {
					order = 3,
					name = L["Icon Scale"],
					desc = L["Icon scaling, this lets you enlarge or shrink your icons on both the World Map and Minimap."],
					type = "range",
					min = 0.5, max = 1, step = 0.05,
					arg = "scale",
				},
				iconAlpha = {
					order = 4,
					name = L["Icon Alpha"],
					desc = L["Icon alpha value, this lets you change the transparency of the icons. Only applies on World Map."],
					type = "range",
					min = 0.1, max = 1, step = 0.05,
					arg = "alpha",
				},
				minimapNodeRange = {
					order = 5,
					type = "toggle",
					name = L["Show Nodes on Minimap Border"],
					width = "double",
					desc = L["Shows more Nodes that are currently out of range on the minimap's border."],
					arg = "nodeRange",
				},
				tracking = {
					order = 6,
					name = L["Tracking Circle Color"],
					desc = L["Color of the tracking circle."],
					type = "group",
					guiInline = true,
					get = function(info)
						local t = db.trackColors[info.arg]
						return t.Red, t.Green, t.Blue, t.Alpha
					end,
					set = function(info, r, g, b, a)
						local t = db.trackColors[info.arg]
						t.Red = r
						t.Green = g
						t.Blue = b
						t.Alpha = a
						Config:UpdateConfig()
					end,
					args = {
						trackingColorMine = {
							order = 5,
							name = L["Mineral Veins"],
							desc = L["Color of the tracking circle."],
							type = "color",
							hasAlpha = true,
							arg = "Mining",
						},
						trackingColorHerb = {
							order = 5,
							name = L["Herb Bushes"],
							desc = L["Color of the tracking circle."],
							type = "color",
							hasAlpha = true,
							arg = "Herb Gathering",
						},
						trackingColorFish = {
							order = 5,
							name = L["Fishing"],
							desc = L["Color of the tracking circle."],
							type = "color",
							hasAlpha = true,
							arg = "Fishing",
						},
						trackingColorGas = {
							order = 5,
							name = L["Gas Clouds"],
							desc = L["Color of the tracking circle."],
							type = "color",
							hasAlpha = true,
							arg = "Extract Gas",
						},
						trackingColorTreasure = {
							order = 6,
							name = L["Treasure"],
							desc = L["Color of the tracking circle."],
							type = "color",
							hasAlpha = true,
							arg = "Opening",							
						},
						space = {
							order = 7,
							name = "",
							desc = "",
							type = "description",
						},
						trackDistance = {
							order = 15,
							name = L["Tracking Distance"],
							desc = L["The distance in yards to a node before it turns into a tracking circle"],
							type = "range",
							min = 50, max = 240, step = 5,
							get = options.get,
							set = options.set,
							arg = "trackDistance",
						},
						trackShow = {
							order = 20,
							name = L["Show Tracking Circle"],
							desc = L["Toggle showing the tracking circle."],
							type = "select",
							get = options.get,
							set = options.set,
							values = prof_options,
							arg = "trackShow",
						},
					},
				},
			},
		},
	},
}

-- Setup some storage arrays by db to sort node names alphabetically
local sortedFilter = setmetatable({}, {__index = function(t, k)
		local new = {}
		for name, id in pairs(GatherMate.nodeIDs[k]) do
			new[name] = name
		end
		rawset(t, k, new)
		return new
	end
})

-- Setup some helper functions
local ConfigFilterHelper = {}
function ConfigFilterHelper:SelectAll(info)
	local db = db.filter[info.arg]
	local nids = GatherMate.nodeIDs[info.arg]
	for k, v in pairs(nids) do
		db[v] = true
	end
	Config:UpdateConfig()
end
function ConfigFilterHelper:SelectNone(info)
	local db = db.filter[info.arg]
	local nids = GatherMate.nodeIDs[info.arg]
	for k, v in pairs(nids) do
		db[v] = false
	end
	Config:UpdateConfig()
end
function ConfigFilterHelper:SetState(info, k, state)
	db.filter[info.arg][GatherMate.nodeIDs[info.arg][k]] = state
	Config:UpdateConfig()
end
function ConfigFilterHelper:GetState(info, k)
	return db.filter[info.arg][GatherMate.nodeIDs[info.arg][k]]
end

local ImportHelper = {}

function ImportHelper:GetImportStyle(info,k)
	return db["importers"][info.arg].Style
end
function ImportHelper:SetImportStyle(info,k,state)
	db["importers"][info.arg].Style = k
end
function ImportHelper:GetImportDatabase(info,k)
	return db["importers"][info.arg].Databases[k]
end
function ImportHelper:SetImportDatabase(info,k,state)
	db["importers"][info.arg].Databases[k] = state
end
function ImportHelper:GetAutoImport(info, k)
	return db["importers"][info.arg].autoImport
end
function ImportHelper:SetAutoImport(info,state)
	db["importers"][info.arg].autoImport = state
end
function ImportHelper:GetBCOnly(info,k)
	return db["importers"][info.arg].bcOnly
end
function ImportHelper:SetBCOnly(info,state)
	db["importers"][info.arg].bcOnly = state
end

local commonFiltersDescTable = {
	order = 0,
	type = "description",
	name = L["Filter_Desc"],
}
options.args.display.args.filters = {
	type = "group",
	name = L["Filters"],
	order = 2,
	--childGroups = "tab", -- this makes the filter tree become inline tabs
	handler = ConfigFilterHelper,
	args = {},
}
options.args.display.args.filters.args.herbs = {
	type = "group",
	name = L["Herb filter"],
	desc = L["Select the herb nodes you wish to display."],
	args = {
		desc = commonFiltersDescTable,
		select_all = {
			order = 1,
			name = L["Select All"],
			desc = L["Select all nodes"],
			type = "execute",
			func = "SelectAll",
			arg = "Herb Gathering",
		},
		select_none = {
			order = 2,
			desc = L["Clear node selections"],
			name = L["Select None"],
			type = "execute",
			func = "SelectNone",
			arg = "Herb Gathering",
		},
		herblist = {
			order = 3,
			name = L["Herb Bushes"],
			type = "multiselect",
			values = sortedFilter["Herb Gathering"],
			set = "SetState",
			get = "GetState",
			arg = "Herb Gathering",
		},
	},
}
options.args.display.args.filters.args.mines = {
	type = "group",
	name = L["Mine filter"],
	desc = L["Select the mining nodes you wish to display."],
	args = {
		desc = commonFiltersDescTable,
		select_all = {
			order = 1,
			name = L["Select All"],
			desc = L["Select all nodes"],
			type = "execute",
			func = "SelectAll",
			arg = "Mining",
		},
		select_none = {
			order = 2,
			desc = L["Clear node selections"],
			name = L["Select None"],
			type = "execute",
			func = "SelectNone",
			arg = "Mining",
		},
		minelist = {
			order = 3,
			name = L["Mineral Veins"],
			type = "multiselect",
			values = sortedFilter["Mining"],
			set = "SetState",
			get = "GetState",
			arg = "Mining",
		},
	},
}
options.args.display.args.filters.args.fish = {
	type = "group",
	name = L["Fish filter"],
	args = {
		desc = commonFiltersDescTable,
		select_all = {
			order = 1,
			name = L["Select All"],
			desc = L["Select all nodes"],
			type = "execute",
			func = "SelectAll",
			arg = "Fishing",
		},
		select_none = {
			order = 2,
			desc = L["Clear node selections"],
			name = L["Select None"],
			type = "execute",
			func = "SelectNone",
			arg = "Fishing",
		},
		fishlist = {
			order = 3,
			name = L["Fishes"],
			type = "multiselect",
			desc = L["Select the fish nodes you wish to display."],
			values = sortedFilter["Fishing"],
			set = "SetState",
			get = "GetState",
			arg = "Fishing",
		},
	},
}
options.args.display.args.filters.args.gas = {
	type = "group",
	name = L["Gas filter"],
	args = {
		desc = commonFiltersDescTable,
		select_all = {
			order = 1,
			name = L["Select All"],
			desc = L["Select all nodes"],
			type = "execute",
			func = "SelectAll",
			arg = "Extract Gas",
		},
		select_none = {
			order = 2,
			name = L["Select None"],
			desc = L["Clear node selections"],
			type = "execute",
			func = "SelectNone",
			arg = "Extract Gas",
		},
		gaslist = {
			order = 3,
			name = L["Gas Clouds"],
			desc = L["Select the gas clouds you wish to display."],
			type = "multiselect",
			values = sortedFilter["Extract Gas"],
			set = "SetState",
			get = "GetState",
			arg = "Extract Gas",
		},
	},
}
options.args.display.args.filters.args.treasure = {
	type = "group",
	name = L["Treasure filter"],
	args = {
		desc = commonFiltersDescTable,
		select_all = {
			order = 1,
			name = L["Select All"],
			desc = L["Select all nodes"],
			type = "execute",
			func = "SelectAll",
			arg = "Opening",
		},
		select_none = {
			order = 2,
			name = L["Select None"],
			desc = L["Clear node selections"],
			type = "execute",
			func = "SelectNone",
			arg = "Opening",
		},
		gaslist = {
			order = 3,
			name = L["Treasure"],
			desc = L["Select the treasure you wish to display."],
			type = "multiselect",
			values = sortedFilter["Opening"],
			set = "SetState",
			get = "GetState",
			arg = "Opening",
		},
	},
}

-- Cleanup config tree
options.args.cleanup = {
 	type = "group",
	name = L["Cleanup Database"],
	order = 5,
	args = {
		desc = {
			order = 0,
			type = "description",
			name = L["Cleanup_Desc"],
		},
		cleanup_range = {
			order = 10,
			name = L["Cleanup radius"],
			type = "group",
			guiInline = true,
			get = function(info)
				return db.cleanupRange[info.arg]
			end,
			set = function(info, v)
				db.cleanupRange[info.arg] = v
			end,
			args = {
				desc = {
					order = 0,
					type = "description",
					name = L["CLEANUP_RADIUS_DESC"],
				},
				Mine = {
					order = 5,
					name = L["Mineral Veins"],
					desc = L["Cleanup radius"],
					type = "range",
					min = 0, max = 30, step = 1,
					arg = "Mining",
				},
				Herb = {
					order = 5,
					name = L["Herb Bushes"],
					desc = L["Cleanup radius"],
					type = "range",
					min = 0, max = 30, step = 1,
					arg = "Herb Gathering",
				},
				Fish = {
					order = 5,
					name = L["Fishes"],
					desc = L["Cleanup radius"],
					type = "range",
					min = 0, max = 30, step = 1,
					arg = "Fishing",
				},
				Gas = {
					order = 5,
					name = L["Gas Clouds"],
					desc = L["Cleanup radius"],
					type = "range",
					min = 0, max = 100, step = 1,
					arg = "Extract Gas",
				},
				Treasure = {
					order = 5,
					name = L["Treasure"],
					desc = L["Cleanup radius"],
					type = "range",
					min = 0, max = 30, step = 1,
					arg = "Opening",					
				}
			},
		},
		cleanup = {
			name = L["Cleanup Database"],
			desc = L["Cleanup your database by removing duplicates. This takes a few moments, be patient."],
			type = "execute",
			handler = GatherMate,
			func = "CleanupDB",
			order = 20,
		},
	},
}


-- GatherMateData Import config tree
options.args.importing = {
 	type = "group",
	name = L["Import Data"],
	order = 10,
	args = {},
}
ImportHelper.db_options = {
	["Merge"] = L["Merge"],
	["Overwrite"] = L["Overwrite"]
}
ImportHelper.db_tables = {
	["Herbs"] = L["Herbalism"],
	["Mines"] = L["Mining"],
	["Gases"] = L["Gas Clouds"],
	["Fish"] = L["Fishing"],
	["Treasure"] = L["Treasure"],
}
imported["GatherMate_Data"] = false
options.args.importing.args.GatherMateData = {
 	type = "group",
	name = "GatherMateData", -- addon name to import from, don't localize
	handler = ImportHelper,
	disabled = function()
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo("GatherMate_Data")
		-- disable if the addon is not enabled, or
		-- disable if there is a reason why it can't be loaded ("MISSING" or "DISABLED")
		return not enabled or (reason ~= nil)
	end,
	args = {
		desc = {
			order = 0,
			type = "description",
			name = L["Importing_Desc"],
		},
		loadType = {
			order = 1,
			name = L["Import Style"],
			desc = L["Merge will add GatherMateData to your database. Overwrite will replace your database with the data in GatherMateData"],
			type = "select",
			values = ImportHelper.db_options,
			set = "SetImportStyle",
			get = "GetImportStyle",
			arg = "GatherMate_Data",
		},
		loadDatabase = {
			order = 2,
			name = L["Databases to Import"],
			desc = L["Databases you wish to import"],
			type = "multiselect",
			values = ImportHelper.db_tables,
			set = "SetImportDatabase",
			get = "GetImportDatabase",
			arg = "GatherMate_Data",
		},
		loadAuto = {
			order = 3,
			name = L["Auto Import"],
			desc = L["Automatically import when ever you update your data module, your current import choice will be used."],
			type = "toggle",
			get = "GetAutoImport",
			set = "SetAutoImport",
			arg = "GatherMate_Data",
		},
		loadBC = {
			order = 4,
			name = L["BC Data Only"],
			desc = L["Only import Burning Crusade data from WoWHead"],
			type = "toggle",
			get  = "GetBCOnly",
			set  = "SetBCOnly",
			arg  = "GatherMate_Data",
		},
		loadData = {
			order = 8,
			name = L["Import GatherMateData"],
			desc = L["Load GatherMateData and import the data to your database."],
			type = "execute",
			func = function() 
				local loaded, reason = LoadAddOn("GatherMate_Data")
				local GatherMateData = LibStub("AceAddon-3.0"):GetAddon("GatherMate_Data")
				if loaded then
					local dataVersion = tonumber(GetAddOnMetadata("GatherMate_Data", "X-Generated-Version"):match("%d+"))
					GatherMateData:PerformMerge(db.importers["GatherMate_Data"].Databases,db.importers["GatherMate_Data"].Style,db.importers["GatherMate_Data"].bcOnly)
					GatherMateData:CleanupImportData()
					Config:Print(L["GatherMateData has been imported."])
					Config:SendMessage("GatherMateConfigChanged")
					db["importers"]["GatherMate_Data"]["lastImport"] = dataVersion
					imported["GatherMate_Data"] = true
				else
					Config:Print(L["Failed to load GatherMateData due to "]..reason)
				end
			end,
			disabled = function()
				local cm = 0
				if db["importers"]["GatherMate_Data"].Databases["Mines"] then cm = 1 end
				if db["importers"]["GatherMate_Data"].Databases["Herbs"] then cm = 1 end
				if db["importers"]["GatherMate_Data"].Databases["Gases"] then cm = 1 end
				if db["importers"]["GatherMate_Data"].Databases["Fish"] then cm = 1 end
				return imported["GatherMate_Data"] or (cm == 0 and not imported["GatherMate_Data"])
			end,
		}
	},
}

options.args.faq_group = {
	type = "group",
	name = L["FAQ"],
	desc = L["Frequently Asked Questions"],
	order = -1,
	args = {
		header = {
			type = "header",
			name = L["Frequently Asked Questions"],
			order = 0,
		},
		desc = {
			type = "description",
			name = L["FAQ_TEXT"],
			order = 1,
		},
	},
}

local getProfilesOptionsTable
do
	local defaultProfiles
	--[[ Utility functions ]]
	-- get exisiting profiles + some default entries
	local tmpprofiles = {}
	local function getProfileList(db, common, nocurrent)
		-- clear old profile table
		local profiles = {}
		
		-- copy existing profiles into the table
		local curr = db:GetCurrentProfile()
		for i,v in pairs(db:GetProfiles(tmpprofiles)) do if not (nocurrent and v == curr) then profiles[v] = v end end
		
		-- add our default profiles to choose from
		for k,v in pairs(defaultProfiles) do
			if (common or profiles[k]) and not (k == curr and nocurrent) then
				profiles[k] = v
			end
		end
		return profiles
	end
	
	function getProfilesOptionsTable(db)
		defaultProfiles = {
			["Default"] = L["Default"],
			[db.keys.char] = L["Char:"] .. " " .. db.keys.char,
			[db.keys.realm] = L["Realm:"] .. " " .. db.keys.realm,
			[db.keys.class] = L["Class:"] .. " " .. UnitClass("player")
		}
		
		local tbl = {
			profiles = {
				type = "group",
				name = L["Profiles"],
				desc = L["Manage Profiles"],
				order = 20,
				args = {
					desc = {
						order = 1,
						type = "description",
						name = L["You can change the active database profile of GatherMate, so you can have different settings and filters for every character, which will allow a very flexible configuration for everyones needs."] .. "\n",
					},
					descreset = {
						order = 9,
						type = "description",
						name = L["Reset the current profile back to its default values, in case your configuration is broken, or you simply want to start over."],
					},
					reset = {
						order = 10,
						type = "execute",
						name = L["Reset Profile"],
						desc = L["Reset the current profile to the default"],
						func = function() db:ResetProfile() end,
					},
					choosedesc = {
						order = 21,
						type = "description",
						name = "\n" .. L["You can create a new profile by entering a new name in the editbox, or choosing one of the already exisiting profiles."],
					},
					new = {
						name = L["New"],
						desc = L["Create a new empty profile."],
						type = "input",
						order = 30,
						get = function() return false end,
						set = function(info, value) db:SetProfile(value) end,
					},
					choose = {
						name = L["Current"],
						desc = L["Select one of your currently available profiles."],
						type = "select",
						order = 40,
						get = function() return db:GetCurrentProfile() end,
						set = function(info, value) db:SetProfile(value) end,
						values = function() return getProfileList(db, true) end,
					},
					copydesc = {
						order = 51,
						type = "description",
						name = "\n" .. L["Copy the settings from one existing profile into the currently active profile."],
					},
					copyfrom = {
						order = 60,
						type = "select",
						name = L["Copy From"],
						desc = L["Copy the settings from another profile into the active profile."],
						get = function() return false end,
						set = function(info, value) db:CopyProfile(value) end,
						values = function() return getProfileList(db, nil, true) end,
					},
					deldesc = {
						order = 70,
						type = "description",
						name = "\n" .. L["Delete existing and unused profiles from the database to save space, and cleanup the GatherMate SavedVariables file."],
					},
					delete = {
						order = 80,
						type = "select",
						name = L["Delete a Profile"],
						desc = L["Deletes a profile from the database."],
						get = function() return false end,
						set = function(info, value) db:DeleteProfile(value) end,
						values = function() return getProfileList(db, nil, true) end,
						confirm = true,
						confirmText = L["Are you sure you want to delete the selected profile?"],
					},
				},
			},
		}
		return tbl
	end
end


--[[
	Initialize the Config System
]]

function Config:OnInitialize()
	db = GatherMate.db.profile
	options.plugins["profiles"] = getProfilesOptionsTable(GatherMate.db)
	self.options = options
	self.importHelper = ImportHelper
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("GatherMate", options)
	self:RegisterChatCommand(L["gathermate"], function() LibStub("AceConfigDialog-3.0"):Open("GatherMate") end )
	self:CheckAutoImport()	
	self:RegisterMessage("GatherMateConfigChanged")
end

function Config:UpdateConfig()
	self:SendMessage("GatherMateConfigChanged")
end

function Config:GatherMateConfigChanged()
	db = GatherMate.db.profile
end

function Config:CheckAutoImport()
	for k,v in pairs(db.importers) do
		local verline = GetAddOnMetadata(k, "X-Generated-Version")
		if verline and v["autoImport"] then 
			local dataVersion = tonumber(verline:match("%d+"))
			if dataVersion and dataVersion > v["lastImport"] then
				local loaded, reason = LoadAddOn(k)
				local addon = LibStub("AceAddon-3.0"):GetAddon(k)
				if loaded then
					addon:PerformMerge(v.Databases,v.Style,v.bcOnly)
					addon:CleanupImportData()
					imported[k] = true
					Config:SendMessage("GatherMateConfigChanged")
					v["lastImport"] = dataVersion
					Config:Print(L["Auto import complete for addon "]..k)
				end
			end
		end
	end
end

-- Allows an external import module to insert their aceopttable into the Importing tree
-- returns a reference to the saved variables state for the addon
function Config:RegisterImportModule(moduleName, optionsTable)
	options.args.importing.args[moduleName] = optionsTable
	return db.importers[moduleName]
end
-- Allows an external module to insert their aceopttable
function Config:RegisterModule(moduleName, optionsTable)
	options.args[moduleName] = optionsTable
end
